/**
 * Tabelka z obiektami po których można przesuwać się przy pomocy
 * strzałek na pilocie.
 */
function Grid(domId, cols, rows, options) {
	var domId = domId, rows = rows, cols = cols;
	var data = null;
	var dataRows = -1;
	var element = null; 
	var elements = null;

	var dataIdx = -1
	var elementIdx = -1
	var elementsSize = -1;

	// zdarzenia związane z poruszaniem się po elementach tablicy
	// - zdarzenia związane z przesunięciem danych
	var onUpShift = null;  // przesunięca do góry
	var onDownShift = null; // przesunięca w dół

	// - osiągnięcie kresu danych
	var onLeftBoundary = null;
	var onRightBoundary = null;
	var onTopBoundary = null;
	var onBottomBoundary = null;

	var onUnload = null; // przed załadowaniem danych np. wyczyszczenie komórek
	var onLoad = null;   // załadowanie danych
	var onFocus = null;  // focus na elemencie
	var onBlur = null;   // usunięcie focusa z elementu

	var scrollHeight = options && options.scrollHeight ? options.scrollHeight : 0;

	// Zwracamy współrzedne x i y dla podanego indeksu.
	function getPos(tmpIdx) {
		tmpIdx = tmpIdx || elementIdx;
		var result = {x: (tmpIdx % cols), y: Math.floor(tmpIdx/cols)};
		logger.debug('Grid(' + domId + ').getPos(' + tmpIdx + '), x:' + result.x + ', y: ' + result.y);
		return result;
	}

	// Przesunięcie danych w wyświetlanej tabelce.
	function shift(dataOffset) {
		for (var i = 0; i < elements.length; i++) {
			if (data[i + dataOffset]) {
				onLoad(elements[i], data[i + dataOffset]);
			} else {
				onUnload(elements[i]);
			}
		}
	}

	this.init = function(elementsClassName, options) {
		options = options || {};

		element = document.getElementById(domId);
		elements = DOMUtil.getElementsByClassName(element, elementsClassName);

		onUpShift = options.onUpShift || function() {};
		onDownShift = options.onDownShift || function() {};

		onLeftBoundary = options.onLeftBoundary || function() {};
		onRightBoundary = options.onRightBoundary || function() {};
		onTopBoundary = options.onTopBoundary || function() {};
		onBottomBoundary = options.onBottomBoundary || function() {};

		onUnload = options.onUnload || function(elem) {};
		onLoad = options.onLoad || function() {elem, data};
		onFocus = options.onFocus || function(elem, data) {};
		onBlur = options.onBlur || function(elem, data) {};

		if (!element) throw 'Błąd, brakuje elementu: ' + domId;
		if (!elements) throw 'Błąd, brakuje elementów listy: ' + elementsClassName;
		if (!onLoad) throw 'Błąd, brakuje funkcji do obsługi ładowania danych (onLoad).';
		if (!onFocus) throw 'Błąd, brakuje funkcji do obsługi wybranego elementu (onFocus).';
		if (!onBlur) throw 'Błąd, brakuje funkcji do obsługi wybranego elementu (onBlur).';
		return this;
	};

	this.load = function(pData, idx) {
		data = pData;
		dataRows = Math.ceil(pData.length/cols);
		elementsSize = elements.length > data.length ? data.length : elements.length;

        return this.reset(idx);
    };

    this.reset = function(idx) {
		idx = (typeof idx == 'undefined') ? -1 : idx;
		dataIdx = elementIdx = idx;
        
		// Czyszczenie komórek po poprzednich danych.
		for (var i = 0; i < elements.length; i++) {
			onUnload(elements[i]);
		}

		for (var i = 0; i < elementsSize; i++) {
			onLoad(elements[i], data[i]);
			if (idx == i) {
				onFocus(elements[i], data[i]);
			}
		}
		scroll();
		return this;
	};

	// Zmiana focusa z poprzednio zaznaczonego elementu na 
	// nowo wybrany.
	function change(i) {
		onBlur(elements[elementIdx], data[dataIdx]);
		elementIdx += i;
		dataIdx += i;
		onFocus(elements[elementIdx], data[dataIdx]);
	}

	// Zdarzenie wciśnięcia lewej strzałki.
	this.left = function() {
		logger.debug('Grid(' + domId + ').left()');
		var pos = getPos();
		if (pos.x > 0 && dataIdx > 0) {
			change(-1);
		} else {
			onLeftBoundary();
		}
	};

	// Zdarzenie wciśnięcia prawej strzałki.
	this.right = function() {
		logger.debug('Grid(' + domId + ').right()');
		var pos = getPos();
		if (pos.x < cols - 1 && dataIdx + 1 < data.length) {
			change(1);
		} else {
			onRightBoundary();
		}
	};

	// Zdarzenie wciśnięcia strzałki do góry.
	this.up = function() {
		logger.debug('Grid(' + domId + ').down()');
		var pos = getPos();
		if (pos.y > 0) {
			change(-cols);
		} else {
			if (dataIdx >= cols) {
				dataIdx -= cols;
				shift(dataIdx - getPos(elementIdx).x);
				onUpShift();
			} else {
				onTopBoundary();
			}
		}
		scroll();
	};

	// Zdarzenie wciśnięcia strzałki w dół.
	this.down = function() {
		logger.debug('Grid(' + domId + ').down()');
		var pos = getPos();
		if (pos.y < rows - 1) {
			if (dataIdx + cols < data.length) {
				change(cols); // przesunięcie wskaźnika o N kolumn
			} else if (dataRows > pos.y) {
				change(data.length - dataIdx - 1); // przesunięcie na ostatni element w ostatnim wierszu
			}
		} else {
			// czy są jeszcze dane na następny wiersz
			if (dataIdx + (cols - pos.x) < data.length) {
				// ustawienie aktywnego elementu na ostatni
				if ((dataIdx + cols) >= data.length) {
					onBlur(elements[elementIdx], data[dataIdx]);
					shift((dataIdx - elementIdx) + cols);
					dataIdx = data.length - 1;
					elementIdx = elementIdx - (getPos(elementIdx).x - getPos(dataIdx).x); 
					onFocus(elements[elementIdx], data[dataIdx]);
				} else {
					dataIdx += cols;
					shift(dataIdx - elementIdx);
				}
				onDownShift();
			} else {
				onBottomBoundary();
			}
		}
		scroll();
	};

	// Zwrócenie danych i elementu DOM dla aktualnie wybranej 
	// komórki w tabelce.
	this.getCurrent = function() {
		return data[dataIdx] ? {element: elements[elementIdx], data: data[dataIdx]} : null;
	};

	// metody do przesuwania się w przód kolekcji
	this.next = function() {
		var pos = getPos(elementIdx);
		if (dataIdx < data.length - 1) {
			if (pos.y == rows - 1 && pos.x == cols -1) {
				onBlur(elements[elementIdx], data[dataIdx]);
				shift((dataIdx - elementIdx) + cols);
				dataIdx += 1;
				elementIdx -= cols - 1;
				onFocus(elements[elementIdx], data[dataIdx]);
			} else {
				change(1);
			}
			scroll();
		}
	};

	// metody do przesuwania się w tył kolekcji
	this.prev = function() {
		var pos = getPos(elementIdx);
		if (dataIdx > 0) {
			if (pos.y == 0 && pos.x == 0) {
				// przesunięcie
				onBlur(elements[elementIdx], data[dataIdx]);
				shift((dataIdx - elementIdx) - cols);
				dataIdx -= 1;
				elementIdx += cols - 1;
				onFocus(elements[elementIdx], data[dataIdx]);
			} else {
				change(-1);
			}
			scroll();
		}
	};

	this.select = function(idx) {
		if (elementIdx > -1 && dataIdx > -1) {
			onBlur(elements[elementIdx], data[dataIdx]);
		}
		if (idx > -1) {
			onFocus(elements[idx], data[idx]);
		}
		elementIdx = idx;
		dataIdx = idx;
		return this;
	};

	this.blur = function() { this.select(-1); };

	this.focus = function() { 
        this.select(0); 
	};

    this.size = function() { return data != null ? data.length : 0 }

	function scroll() {
		logger.debug('Grid.scroll()');
		var scrollElement = document.getElementById(domId + '-scroll');
		var scrollMarkElement = document.getElementById(domId + '-scroll-mark');
		var scrollMarkFillElement = document.getElementById(domId + '-scroll-mark-mid');
        scrollMarkFillElement.style.height = ((dataRows > 0) ? ((scrollHeight / dataRows) - 10) : 0) + 'px';
		var y = getPos(dataIdx).y;
		var position;
		if (scrollElement) {
			logger.debug('getPos(dataIdx).y: ' + y + ', scroll.clientHeight: ' + scrollHeight + ', rows: ' + dataRows);
			if (dataRows <= rows) {
				logger.debug('Grid.scroll(), hide');
				DOMUtil.hide(scrollElement);
			} else {
				if (y <= 0) {
					position = 0;
				} else {
					position = y * (scrollHeight/dataRows);
				}
				logger.debug('Setting scroll position: ' + position);
				scrollMarkElement.style.top = position + 'px';
				DOMUtil.show(scrollElement);
			}
		}
	};
}
