/**
 * Komponent do obsługi listy danych
 */
function List(domId) {
	var domId = domId;
	var data = null;
	var element = null; 
	var elements = null;

	var dataIdx = -1;
	var elementIdx = -1;
	var elementsSize = -1;
    var maxElementsSize = -1; // ile maksymalnie elementów może być utworzonych dynamicznie

	var onLoad = onUnload = null;
	var onFocus = onBlur = null;
	var onTopBoundary = null;
	var onBottomBoundary = null;
	var onOverflow = null;

	this.init = function(elementsClassName, options) {
		options = options || {};

        maxElementsSize = options.maxElementsSize || -1;

		element = document.getElementById(domId);
		elements = DOMUtil.getElementsByClassName(element, elementsClassName);

		onUnload = options.onUnload || function(elem) {};
		onLoad = options.onLoad || function(elem, data) {};
		onFocus = options.onFocus || function() {elem, data};
		onBlur = options.onBlur || function() {elem, data};

		onTopBoundary = options.onTopBoundary || function() {};
		onBottomBoundary = options.onBottomBoundary || function() {};

		onOverflow = options.onOverflow || function() {};


		if (!element) throw 'Błąd, brakuje elementu: ' + domId;
		if (!elements) throw 'Błąd, brakuje elementów listy: ' + elementsClassName;
		if (!onLoad) throw 'Błąd, brakuje funkcji do obsługi ładowania danych (onLoad).';
		if (!onFocus) throw 'Błąd, brakuje funkcji do obsługi wybranego elementu (onFocus).';
		if (!onBlur) throw 'Błąd, brakuje funkcji do obsługi wybranego elementu (onBlur).';
		return this;
	};

	this.load = function(pData, idx) {
		logger.debug('List: ' + domId + ' load');
		data = pData;


        if (elements.length == 0 && maxElementsSize > 0) {
            for (var i = 0; i < maxElementsSize; i++) {
                elements.push(DOMUtil.addChild(element, 'li', 'item'));
            }
        }

		elementsSize = elements.length > data.length ? data.length : elements.length;
        logger.debug('elementsSize: ' + elementsSize);

		idx = (typeof idx == 'undefined') ? -1 : idx;
		dataIdx = elementIdx = idx;

		for (var i = 0; i < elements.length; i++) {
			onUnload(elements[i]);
		}

		for (var i = 0; i < elementsSize; i++) {
			onLoad(elements[i], data[i]);
			if (idx == i) {
				onFocus(elements[i], data[i]);
			}
		}

		if (onOverflow && data.length > elementsSize) {
			onOverflow(false, true);
		}
		return this;
	};

	function signalOverflow(idx) {
		if (onOverflow) {
            logger.debug('idx: ' + idx + ', elementIdx: ' + elementIdx + ', elementsSize: ' + elementsSize + ', data.length: ' + data.length);
            onOverflow((0 < idx - elementIdx), (data.length > (idx + (elementsSize - elementIdx))));
		}
	}

	this.prev = function() {
		if (dataIdx > 0) {
			dataIdx -= 1;
			if (elementIdx > 0) {
				onBlur(elements[elementIdx--]);
				onFocus(elements[elementIdx]);
			} else {
				for (var i = 0; i < elementsSize; i++) {
					onLoad(elements[i], data[dataIdx + i]);
				}
				signalOverflow(dataIdx);
			}
		} else {
			onTopBoundary();
		}
		return this;
	};

	this.next = function() {
		if (dataIdx < data.length - 1) {
			dataIdx += 1;
			if (elementIdx < elementsSize -1) {
				onBlur(elements[elementIdx++]);
				onFocus(elements[elementIdx]);
			} else {
				for (var i = 0; i < elementsSize; i++) {
					onLoad(elements[i], data[dataIdx - elementsSize + i + 1]);
				}
				signalOverflow(dataIdx);
			}
		} else {
			onBottomBoundary();
		}
		return this;
	};

	this.getCurrent = function() {
		return (data != null && data[dataIdx]) ? {element: elements[elementIdx], data: data[dataIdx]} : null;
	};

	this.focus = function() {
		if (elementIdx == -1) {
			elementIdx = 0;
		   	dataIdx = 0;
		}
		logger.debug('List(' + name + ').focus, elementIdx: ' + elementIdx + ', dataIdx: ' + dataIdx);
		onFocus(elements[elementIdx]);
	};

	this.blur = function() {
		onBlur(elements[elementIdx]);
	};

	this.isEmpty = function() {
		return (data == null || data.length == 0);
	};

	this.removeCurrent = function() {
		if (dataIdx > -1) {
			data.splice(dataIdx, 1);
			if (data.length == 0) {
				onUnload(elements[elementIdx]);
				dataIdx = -1;
				elementIdx = -1;
				this.blur();
			} else {
				if (data[dataIdx]) {
					// przesunięcie elementów z prawej w lewo
					for (var i = 0; elementIdx + i < elementsSize; i++) {
						if (data[dataIdx + i]) {
							onLoad(elements[elementIdx + i], data[dataIdx + i]);
						} else {
							onUnload(elements[elementIdx + i]);
						}	
					}
				} else {
					if (elementIdx > 0) {
						onBlur(elements[elementIdx]);
						onUnload(elements[elementIdx]);
						dataIdx -= 1;
						elementIdx -= 1;
						onFocus(elements[elementIdx]);
					} else {
						dataIdx -= 1;
						onLoad(elements[elementIdx], data[dataIdx]);
					}
				}
			}

			// TODO: ustawienie znaczników przepełnienia
			signalOverflow(dataIdx);
		}
	};
}


