var Player = {
	domId: null,
    plugin: null,
	controls: {},
	fcontrols: {},
    mwPlugin: null,
    nnaviPlugin: null,
	totalTime: -1,
	currentTime: 0,
    state: -1,
    skipState: -1,
    mode: -1,
    videoId: -1,
    customInfo: [],
    stopCallback: null,
    originalSource: null,
	start: { x: 20, y: 123, },
	width: 601, 
	height: 338,

    STOPPED: 0,
    PLAYING: 1,
    PAUSED: 2,  
    FORWARD: 3,  
    REWIND: 4,  

	HIDDEN: 0,
	WINDOW: 1,
	FULLSCREEN: 2,

	// callbacki
	onStop: function() {},

	msg: {
		1: 'Nieobsługiwany format.',
		2: 'Nieobsługiwany kodek video.',
		3: 'Nieobsługiwany kodek audio.',
		4: 'Nieobsługiwana rozdzielczość video.'
	},
};

Player.init = function(domId) {
	var result = false;
	this.state = this.STOPPED;
	this.domId = domId;
	
	this.plugin = document.getElementById("pluginVideo");
	this.mwPlugin = document.getElementById("pluginTVMW");

	if (this.plugin && this.mwPlugin) {
		this.originalSource = this.mwPlugin.GetSource();
		this.mwPlugin.SetMediaSource();
        this.plugin.SetTotalBufferSize(2*1024*1024);
        this.plugin.SetInitialBuffer(30);
        this.plugin.SetInitialTimeOut(15);

		result = true;
	}
	
	this.controls = {
		play: document.getElementById(this.domId + '-controls-play'),
		stop: document.getElementById(this.domId + '-controls-stop'),
		rewind: document.getElementById(this.domId + '-controls-rewind'),
		forward: document.getElementById(this.domId + '-controls-forward'),
		buffer: document.getElementById(this.domId + '-buffer-info'),
	};

	this.fcontrols = {
		play: document.getElementById('fullscreen-player-controls-play'),
		stop: document.getElementById('fullscreen-player-controls-stop'),
		rewind: document.getElementById('fullscreen-player-controls-rewind'),
		forward: document.getElementById('fullscreen-player-controls-forward'),
		buffer: document.getElementById('fullscreen-player-buffer-info'),
	};

	this.plugin.OnRenderingComplete = 'Player.renderingComplete';
	this.plugin.OnStreamInfoReady = 'Player.getInfo';
    this.plugin.OnRenderError = 'Player.showError';
	this.plugin.OnCurrentPlayTime = 'Player.setTime';
	this.plugin.OnBufferingStart = 'Player.onBufferingStart';
	this.plugin.OnBufferingComplete = 'Player.onBufferingComplete';
	this.plugin.OnStreamNotFound = 'Player.showVideoNotFoundError';
	this.plugin.OnNetworkDisconnected = 'Player.showNetworkError';
	this.plugin.OnConnectionFailed = 'Player.showNetworkError';
	return result;
};

Player.setScreenSaver = function (state, second) {
	var np = document.getElementById("pluginNNAVI");
    if (!np || np.GetPluginInfo(PL_CMN_INFO_VERSION) < 'NNAVI-0004') {
        return;
    }
                 
    if (state === true) {   // on
        if (!second) {
            var tp = this.mwPlugin;
            if (!tp) return;
            var pf = tp.GetProfile(PL_PRFID_AUTO_PROTECTION_TIME);
            logger.debug('ScreenSaver profile: ' + pf);
            switch (parseInt(pf)) {
                case 0: //PROFILE_DURATION_10MIN :
                    second = 10 * 60;
                    break;
                case 1: //PROFILE_DURATION_20MIN :
                    second = 20 * 60;
                    break;
                case 2: //PROFILE_DURATION_40MIN :
                    second = 40 * 60;
                    break;
                case 3: //PROFILE_DURATION_1HOUR :
                    second = 1 * 60 * 60;
                    break;
                case 4: //PROFILE_DURATION_2HOUR :
                    second = 2 * 60 * 60;
                    break;
                case 5: //PROFILE_DURATION_4HOUR :
                    second = 4 * 60 * 60;
                    break;
                case 6: //PROFILE_DURATION_8HOUR :
                    second = 8 * 60 * 60;
                    break;
                case 7: //PROFILE_DURATION_10HOUR :
                    second = 10 * 60 * 60;
                    break;
                case 8: //PROFILE_DURATION_ALWAYS :
                    // do nothing
                    return;
                default:
                    second = 1 * 60 * 60;
                    break;
            }
        }
        np.SendEventToDevice(3, second);
    } else {  // off
        np.SendEventToDevice(4, 0);
    }
};


Player.renderingComplete = function() {
	logger.debug('Player.renderingComplete()');
    GS.complete(this.currentTime);
	Player.stopVideo(true);
};

Player.getInfo = function() {
	this.totalTime = Player.plugin.GetDuration();
    GS.OnLoadMaterial(this.videoId, ((this.totalTime > 0) ? Math.round(this.totalTime/1000) : 0), this.customInfo);
};

Player.showVideoNotFoundError = function() {
    logger.error('Player:showVideoNotFoundError: połączenie przerwane');
    Main.UI.dialog.error('Film nie został znaleziony');
    Player.stopVideo(true);
};

Player.showNetworkError = function() {
    logger.error('Player:showNetworkError: połączenie przerwane');
    Main.UI.dialog.error('Połączenie zostało przerwane');
    Player.stopVideo(true);
};

Player.showError = function(code) {
	if (this.msg[code]) {
		Main.UI.dialog.error('Błąd VideoPlayera: ' + this.msg[code])
	} else {
		logger.error('Player:OnRenderError: nieznany kod błędu: ' + code);
	}
};

Player.deinit = function() {
    Player.stopVideo(true);
	if (this.mwPlugin && (this.originalSource != null)) {
		this.mwPlugin.SetSource(this.originalSource);
	}
};

Player.onBufferingStart = function() {
    switch (this.skipState) {
        case this.FORWARD:
			if (this.controls.forward != null) {
				this.controls.forward.style.opacity = 1.0;
			}
			if (this.fcontrols.forward != null) {
				this.fcontrols.forward.style.opacity = 1.0;
			}
            break;
        
        case this.REWIND:
			if (this.controls.rewind != null) {
				this.controls.rewind.style.opacity = 1.0;
			}
			if (this.fcontrols.rewind != null) {
				this.fcontrols.rewind.style.opacity = 1.0;
			}
            break;
    }
    DOMUtil.show(this.controls.buffer, true);
    DOMUtil.show(this.fcontrols.buffer, true);
};

Player.onBufferingComplete = function() {
    switch (this.skipState) {
        case this.FORWARD:
			if (this.controls.forward != null) {
				this.controls.forward.style.opacity = 0.2;
			}
			if (this.fcontrols.forward != null) {
				this.fcontrols.forward.style.opacity = 0.2;
			}
            break;
        
        case this.REWIND:
			if (this.controls.rewind != null) {
				this.controls.rewind.style.opacity = 0.2;
			}
			if (this.fcontrols.rewind != null) {
				this.fcontrols.rewind.style.opacity = 0.2;
			}
            break;
    }
    DOMUtil.hide(this.controls.buffer, true);
    DOMUtil.hide(this.fcontrols.buffer, true);
};

Player.setWindowMode = function() {
	this.mode = this.WINDOW;
	logger.debug('Player.setWindowMode > x: ' + this.start.x + ', y: ' + this.start.y + ', width: ' + this.width + ', height: ' + this.height);
	this.plugin.SetDisplayArea(this.start.x, this.start.y, this.width, this.height);
    this.setPlayMode();
};

Player.setFullscreenMode = function() {
	this.mode = this.FULLSCREEN;
	this.plugin.SetDisplayArea(0, 0, 960, 540);
    this.setPlayMode();
};

Player.setHiddenMode = function() {
	this.mode = this.HIDDEN;
	this.plugin.SetDisplayArea(0, 0, 0, 0);
};

Player.setPlayMode = function() {
    if (this.state == this.PAUSED) {
        DOMUtil.toggleClassName(this.controls.play, 'pause', 'play');
        DOMUtil.toggleClassName(this.fcontrols.play, 'pause', 'play');
        this.state = this.PLAYING;
        Player.setScreenSaver(false);
    }
};

Player.setVideoURL = function(url, title, videoId) {
	this.url = url;
    this.videoId = videoId;
    this.customInfo = [{name: 'TITLE', value: title}, {name: 'PLATFORM', value: 'Samsung'}];
};

Player.playVideo = function() {
	if (this.url != null) {
		if (this.state == this.PLAYING) {
			//this.pauseVideo();
		} else if (this.state == this.PAUSED) {
            GS.Play(this.currentTime);
			this.resumeVideo();
		} else {
            GS.Play(this.currentTime);
			this.state = this.PLAYING;
			if (this.mode == this.WINDOW) {
				// workaround: jeśli nie uda się załadować następnego video i kliknie play w starym video
				// na ekranie pojawia się player w dziwnych wymiarach - SYTUACJA TYLKO NA TELEWIZORZE
				this.setWindowMode(); 
			}
			DOMUtil.toggleClassName(this.controls.play, 'pause', 'play');
			DOMUtil.toggleClassName(this.fcontrols.play, 'pause', 'play');
			this.plugin.Play(this.url);
		}
        Player.setScreenSaver(false);
	} else {
		logger.debug("No video to play");
	}
};

Player.pauseVideo = function() {
	if (this.state == this.PAUSED) {
		// this.resumeVideo();
	} else if (this.state == this.PLAYING) {
        GS.Pause(this.currentTime);
		this.state = this.PAUSED;
		this.plugin.Pause();
        DOMUtil.toggleClassName(this.controls.play, 'play', 'pause');
        DOMUtil.toggleClassName(this.fcontrols.play, 'play', 'pause');
	} else {
		logger.debug("Ignoring pause request, not in correct state");
	}
};

Player.stopVideo = function(dontReportInStats) {
	if (this.state != this.STOPPED) {
        Player.setScreenSaver(true);
        if (dontReportInStats === undefined) {
            GS.Stop(this.currentTime);
        }
		this.state = this.STOPPED;
		this.plugin.Stop();
		this.setTime(0);

		DOMUtil.toggleClassName(this.controls.play, 'play', 'pause');
		DOMUtil.toggleClassName(this.fcontrols.play, 'play', 'pause');
		//DOMUtil.toggleClassName(this.controls.stop, 'inactive-ctrl', 'active-ctrl');
		if (this.controls.forward != null) {
			DOMUtil.toggleClassName(this.controls.forward, 'inactive-ctrl', 'active-ctrl');
		}
		if (this.fcontrols.forward != null) {
			DOMUtil.toggleClassName(this.fcontrols.forward, 'inactive-ctrl', 'active-ctrl');
		}
		if (this.controls.rewind != null) {
			DOMUtil.toggleClassName(this.controls.rewind, 'inactive-ctrl', 'active-ctrl');
		}
		if (this.fcontrols.rewind != null) {
			DOMUtil.toggleClassName(this.fcontrols.rewind, 'inactive-ctrl', 'active-ctrl');
		}
		if (this.onStop) {
			this.onStop();
		}
	} else {
		logger.debug("Ignoring stop request, not in correct state");
	}
};

Player.resumeVideo = function() {
	this.state = this.PLAYING;
	this.plugin.Resume();
	DOMUtil.toggleClassName(this.controls.play, 'pause', 'play');
	DOMUtil.toggleClassName(this.fcontrols.play, 'pause', 'play');
};

Player.skipForwardVideo = function() {
	this.skipState = this.FORWARD;
	this.plugin.JumpForward(15);
};

Player.skipBackwardVideo = function() {
	this.skipState = this.REWIND;
	this.plugin.JumpBackward(15);
};

Player.setTime = function(time) {
    this.currentTime = (time > 0) ? Math.round(time/1000) : 0;
    var timePercent = (100 * time) / this.totalTime;
    var timeElement = document.getElementById("video-player-time-info");
    var ftimeElement = document.getElementById("fullscreen-player-time-info");
    var timeHTML = "";
    var timeHour = 0; var timeMinute = 0; var timeSecond = 0;
    var totalTimeHour = 0; var totalTimeMinute = 0; var totalTimesecond = 0;
    
    document.getElementById('video-player-time-progress-bg').style.width = timePercent + "%";
    document.getElementById('fullscreen-player-time-progress-bg').style.width = timePercent + "%";
    
    if (Player.state == Player.STOPPED) {
        timeHTML = "0:00:00/0:00:00";     
    } else {
        totalTimeHour = Math.floor(this.totalTime/3600000);
        timeHour = Math.floor(time/3600000);
        
        totalTimeMinute = Math.floor((this.totalTime%3600000)/60000);
        timeMinute = Math.floor((time%3600000)/60000);
        
        totalTimeSecond = Math.floor((this.totalTime%60000)/1000);
        timeSecond = Math.floor((time%60000)/1000);
        
        timeHTML = timeHour + ":";
        
        if (timeMinute == 0) {
            timeHTML += "00";
		} else if (timeMinute <10) {
            timeHTML += "0" + timeMinute;
		} else {
            timeHTML += timeMinute;
		}
		timeHTML += ':';
            
        if (timeSecond == 0) {
            timeHTML += "00";
		} else if (timeSecond <10) {
            timeHTML += "0" + timeSecond;
		} else {
            timeHTML += timeSecond;
		}
            
        timeHTML += '/' + totalTimeHour + ':';
        
        if (totalTimeMinute == 0) {
            timeHTML += "00";
		} else if (totalTimeMinute <10) {
            timeHTML += "0" + totalTimeMinute;
		} else {
            timeHTML += totalTimeMinute;
		}
        timeHTML += ':';    

        if (totalTimeSecond == 0) {
            timeHTML += "00";
		} else if (totalTimeSecond <10) {
            timeHTML += "0" + totalTimeSecond;
		} else {
            timeHTML += totalTimeSecond;
		}
	}
    
	Main.UI.put(timeElement, timeHTML);
	Main.UI.put(ftimeElement, timeHTML);
};
