var Search = {
	domId: null,
	mode: -1,
	buttonIdx: -1,
	characterIdx: -1,
	setCharacterCallback: null,
	lastChar: '',
	query: '',
	characters: [
		[[' '], [], ['a', 'ą', 'b', 'c', 'ć'], ['d', 'e', 'ę', 'f'], ['g', 'h', 'i'], ['j', 'k', 'l', 'ł'], ['m', 'n', 'ń', 'o', 'ó'], ['p', 'r', 's', 'ś'], ['t', 'u', 'v'], ['w', 'x', 'y', 'z', 'ż', 'ź']],
		[[' '], [], ['A', 'Ą', 'B', 'C', 'Ć'], ['D', 'E', 'Ę', 'F'], ['G', 'H', 'I'], ['J', 'K', 'L', 'Ł'], ['M', 'N', 'Ń', 'O', 'Ó'], ['P', 'R', 'S', 'Ś'], ['T', 'U', 'V'], ['W', 'X', 'Y', 'Z', 'Ż', 'Ź']],
		[['0'], ['1'], ['2'], ['3'], ['4'], ['5'], ['6'], ['7'], ['8'], ['9']],
	],
	labels: [
		['&#9251;', '', 'abc','def', 'ghi', 'jkl', 'mno', 'prs', 'tuv', 'wxyz'],
		['&#9251;', '', 'ABC','DEF', 'GHI', 'JKL', 'MNO', 'PRS', 'TUV', 'WXYZ'],
		['', '', '', '', '', '', '', '', '', ''],
	],

	LOWERCASE_MODE: 0,
	UPPERCASE_MODE: 1,
	NUMBERS_MODE: 2,

	elements: {},
}

Search.init = function(domId) {
	this.domId = domId;
	this.size = this.characters[0].length;
	this.elements = {
		hint: DOMUtil.getElement(domId + '-hint'),
		input: DOMUtil.getElement(domId + '-input'),
	};
};

Search.nextMode = function() {
	Search.setMode((Search.mode == 2 ? 0 : Search.mode + 1), Search.mode);
};

Search.setMode = function(mode, prevMode) {
	if (mode == Search.LOWERCASE_MODE || mode == Search.UPPERCASE_MODE || mode == Search.NUMBERS_MODE) {
		if (typeof prevMode == 'number') {
			FX.Search.blurModeButton(this.domId + '-mode-' + prevMode);
		}
		FX.Search.focusModeButton(this.domId + '-mode-' + mode);
		this.mode = mode;
		this.refreshLabels();
	} else {
		logger.warn('Unkown search mode');
	}
};

Search.refreshLabels = function() {
	for (var i = 0; i < this.size; i++) {
		FX.Search.putLabel(this.domId + '-' + i + '-label', Search.labels[Search.mode][i]);
	}
};

Search.showHint = function(buttonIdx, characterIdx) {
	var text = '', tmp = '';
	var characters = this.characters[Search.mode][buttonIdx];
	if (characters) {
		for (var i = 0; i < characters.length; i++) {
			text += FX.Search.hintElement(characters[i], (i == characterIdx));
		}
	}
	FX.Search.showHints(this.elements.hint, text);
};

Search.hideHint = function() {
	FX.Search.hideHints(this.elements.hint, '');
};

Search.push = function(buttonIdx) {
	clearTimeout(this.setCharacterCallback);
	FX.Search.clickButton(this.domId + '-' + buttonIdx + '-button');
	if (buttonIdx != this.buttonIdx) {
		if (this.buttonIdx != -1) {
			Search.updateQuery(this.lastChar, true);
		}
		this.buttonIdx = buttonIdx;
		this.characterIdx = 0;
	} else {
		// kolejne kliknięcie tego samego guzika
		var characters = Search.characters[Search.mode][buttonIdx];
		if (this.characterIdx + 1 == characters.length) {
			this.characterIdx = 0;
		} else {
			this.characterIdx += 1;
		}
	}
	this.showHint(buttonIdx, this.characterIdx);
	this.lastChar = Search.getCharacter(buttonIdx, this.characterIdx);
	this.updateQuery(this.lastChar, false);
	this.setCharacterCallback = setTimeout('Search.setCharacter()', 1000);
};

Search.pop = function() {
	this.query = this.query.substr(0, this.query.length - 1);
	Main.UI.put(this.elements.input, this.query);
	Search.reset();
};

Search.getCharacter = function(buttonIdx, characterIdx) {
	var characters = this.characters[this.mode][buttonIdx];
	return (characters) ? (characters[characterIdx] ? characters[characterIdx] : '') : '';
};

Search.setCharacter = function() {
	Search.updateQuery(this.lastChar, true);
	Search.reset();
};

Search.reset = function() {
	clearTimeout(Search.setCharacterCallback);
	Search.setCharacterCallback = null;
	Search.buttonIdx = -1;
	Search.characterIdx = -1;
	Search.hideHint();
};

Search.updateQuery = function(character, persist) {
	var tmp = FX.Search.queryElement(character, true);
	if (persist) {
		tmp = '';
		this.query += character;
	}

	Main.UI.put(this.elements.input, (this.query + tmp));
};
