function reportMissingElements() {
	var missing = '';
	for (var i = 0, len = arguments.length; i < len; i++) {
		missing += arguments[i] + ', ';
	}
	if (missing != '') throw 'Błąd! W dokumencie muszą być zdefiniowane elementy: ' + missing;
}

function clone(object) {
	function F() {}
	F.prototype = object;
	F.prototype._super = object;
	return new F;
}

function extend(subClass, superClass) {
	var F = function() {};
	F.prototype = superClass.prototype;
	subClass.prototype = new F();
	subClass.prototype.constructor = subClass;
	subClass.superclass = superClass.prototype;
	if (superClass.prototype.constructor == Object.prototype.constructor) {
		superClass.prototype.constructor = superClass;
	}
}


function mixin(receivingClass, givingClass) {
	if (arguments[2]) {
		for (var i = 2, len = arguments.length; i < len; i++) {
			receivingClass.prototype[arguments[i]] = givingClass.prototype[arguments[i]];
		}
	} else { // Give all methods.
		for (methodName in givingClass.prototype) {
			if (!receivingClass.prototype[methodName]) {
				receivingClass.prototype[methodName] = givingClass.prototype[methodName];
			}
		}
	}
}

String.prototype.trim = function() { return this.replace(/^\s+|\s+$/g, ''); };
Array.prototype.contains = function(obj) {
    var i = this.length;
    while (i--) {
        if (this[i] == obj) {
            return true;
        }
    }
    return false;
};


/**
 * Komunikacja po http z serwerem.
 */
var XHRUtil = {
	sendTimer: null,
    req: null,
    cancel: function() {
		if (XHRUtil.sendTimer != null) {
			clearTimeout(XHRUtil.sendTimer);
		}
        if (XHRUtil.req != null) {
            XHRUtil.req.abort();
        }
		Main.UI.progress.stop();
    },
	send: function(options) {
        XHRUtil.cancel();
		XHRUtil.sendTimer = setTimeout(function() {
		   XHRUtil._send(options);
		}, 0);
	},
	_send: function(options) {
		XHRUtil.sendTimer = null;
		options = {
			type: options.type || 'xml',
			url: options.url ? Main.getUrl(options.url) : '',
			allowedActions: options.allowedActions,
			onSuccess: options.onSuccess || function(){},
			onError: options.onError || function(){},
			onComplete: options.onComplete || function(){},
		};

        if (XHRUtil.req == null) {
            XHRUtil.req = new XMLHttpRequest();
            XHRUtil.req.timeout = 20000;
        }

        XHRUtil.req.onreadystatechange = function() {
            if (XHRUtil.req.readyState == 4) {
                var isSuccess = false;
                var xml;
                if (XHRUtil.req.status == 200 || XHRUtil.req.status == 304 || XHRUtil.req.status == 0) {
                    xml = (options.type == 'xml') ? XHRUtil.req.responseXML : XHRUtil.req.responseText;
                    if (xml) {
                        isSuccess = true;
                        options.onSuccess(xml);
                    }
                }
                if (!isSuccess) {
                    logger.error('Zły status: ' + XHRUtil.req.status + ' (' + XHRUtil.req.statusText + ') lub niepoprawny xml: ' + xml);
                    options.onError('Błąd połączenia.');
                }
                options.onComplete(isSuccess);
            }
        };
        logger.debug('getting url: ' + options.url);
        logger.debug('options.allowedActions: ' + options.allowedActions);
		Main.UI.progress.start(options.allowedActions);
		XHRUtil.req.open('GET', options.url, true);
		XHRUtil.req.send(null);
	},
};

var DOMUtil = {

    addChild: function(parentElement, tagName, className) {
        var element = this.getElement(parentElement);
        var tmp = document.createElement(tagName);
        tmp.setAttribute('class', className);
        element.appendChild(tmp);
        return tmp;
    },

	getChildText: function(element, defaultValue) {
		var result = defaultValue;
		if (element != null && element[0] && element[0].childNodes && element[0].childNodes[0]) {
			result = element[0].childNodes[0].nodeValue;
		}
		return result;
	},

	getChildAttribute: function(element, name, defaultValue) {
		var result = defaultValue;
		if (element && element[0] && element[0].getAttribute(name)) {
				result = element[0].getAttribute(name);
		}
		return result;
	},

	addClassName: function(element, className) {
		element = this.getElement(element);
		if (element) {
			var arr = element.className.split(' ');
			if (arr.indexOf(className) == -1) {
				arr.push(className);
			}
			element.className = arr.join(' ');
		} else {
			logger.debug('Brak elementu: ' + element);
		}
	},

	removeClassName: function(element, className) {
		element = this.getElement(element);
		if (element) {
			if (element.className) {
				var arr = element.className.split(' ');
				var out = [];
				for (var i = 0; i < arr.length; i++) {
					if (arr[i] != className) {
						out.push(arr[i]);
					}
				}
				element.className = out.join(' ');
			}
		} else {
			logger.debug('Brak elementu: ' + element);
		}
	},

    hasClassName: function(element, className) {
        element = this.getElement(element);
        if (element && element.className) {
            var arr = element.className.split(' ');
            if (arr.indexOf(className) > -1) {
                return true;
            }
        }
        return false;
    },

	toggleClassName: function(element, oldClassName, newClassName) {
		DOMUtil.removeClassName(element, oldClassName);
		DOMUtil.addClassName(element, newClassName);
	},
	
	switchClassName: function(fromElement, toElement, className) {
		this.removeClassName(fromElement, className);
		this.addClassName(toElement, className);
	},

	getElementsByClassName: function(node, className) {
		node = this.getElement(node);
		var elements = [];
		var all = node.getElementsByTagName('*');
		//logger.debug('all children size: ' + all.length);
		var regexp = new RegExp("(^|\\s)" + className + "(\\s|$)");
		for (var i = 0, j = all.length; i < j; i++) {
			if (all[i].nodeType == 1 && regexp.test(all[i].className)) {
				elements.push(all[i]);
			}
		}
		return elements;
	},

	getSiblings: function(node, tagName) {
		var elements = [];
		node = this.getElement(node);
		var all = node.parentNode.childNodes;
		for (var i = 0, j = all.length; i < j; i++) {
			if (!this.isIgnorable(all[i]) && all[i].nodeName.toLowerCase() == 'div' && all[i] != node) {
				elements.push(all[i]);
			}
		}
		return elements;
	},

	getLink: function(parentElement) {
		var links = parentElement.getElementsByTagName('a');
		return (links.length > 0) ? links[0] : null;
	},
	
	hide: function(node) {
		logger.debug('DOMUtil.hide(' + node + ')');
		var el = this.getElement(node);
		if (el && el.style) { el.style.display = 'none' }
		return this;
	},
	
	empty: function(node) {
		var el = this.getElement(node);
		if (el) { el.innerHTML = '' }
		return this;
	},

	show: function(nodeId, isBlock) {
        var tmp = (isBlock === undefined) ? '' : 'block'; 
		logger.debug('DOMUtil.show(' + nodeId + '): ' + tmp);
		var el = this.getElement(nodeId);
        logger.debug('show: ' + Utils.objectInfo(el));
		if (el) { el.style.display = tmp }
		return this;
	},

	showAndHideSiblings: function(nodeId) {
		var el = this.getElement(nodeId);
		var tmp = DOMUtil.getSiblings(el);
		for (var i = 0, j = tmp.length; i < j; i++) {
			DOMUtil.hide(tmp[i]);
		}
		DOMUtil.show(el);
	 },

	hideAndShowSiblings: function(nodeId) {
		var el = this.getElement(nodeId);
		var tmp = DOMUtil.getSiblings(el);
		for (var i = 0, j = tmp.length; i < j; i++) {
			DOMUtil.show(tmp[i]);
			break;
		}
		DOMUtil.hide(el);
	},

	visible: function(nodeId) {
		var el = this.getElement(nodeId);
		return el.style.display != 'none';
	 },

	flipVisibility: function(toHideId, toShowId) {
		this.hide(toHideId);
		this.show(toShowId);
	},
	
	getElement: function(el) {
		return (typeof el == 'string') ? document.getElementById(el) : el;
	},

	getPrevious: function(el) {
		el = this.getElement(el);
		while (el = el.previousSibling) {
			if (!this.isIgnorable(el)) { return el; }
		}
		return null;
	},
	
	isIgnorable: function(node) {
		return (node.nodeType == 8) || // A comment node
         ((node.nodeType == 3) && this.isAllWhitespace(node)); // a text node, all ws
	},

	isAllWhitespace: function(node) {
		return !(/[^\t\n\r ]/.test(node.data));
	},

};

var Utils = {
	parseSearch: function(search) {
		var arr = search.substr(1).split("&");
		var result = {};
		var tmp;

		for (var i = 0; i < arr.length; i++) {
			tmp = arr[i].split('=');
			if (tmp[0]) { tmp[0] = unescape(tmp[0].replace(/\+/g," ")); }
			if (tmp[1]) { tmp[1] = unescape(tmp[1].replace(/\+/g," ")); }
			result[tmp[0]] = tmp[1];
		}
		return result;
	},

	objectInfo: function(object) {
		var result = (typeof object);
		for (var propName in object) {
			if (typeof object[propName] != "undefined") {
				 result += (propName + ": " + object[propName] + ', ');
			}
		}
		return result;
	},
					
	
};
