/**
 * Obiekt odpowiedzialny za:
 * 1. zarządzanie widokami (przełączanie się między nimi)
 * 2. interakcje z użytkownikiem
 * 3. komunikacje z serwerem
 * Z serwera zwracane są dokumenty xml'owe w postaci:
 * 	 <object id="" url="" view="">
 * 		<!-- dodatkowe dane -->
 * 		<title>Tytuł</title>
 * 		<description>Opis</description>
 * 		<image></image>
 * Gdzie:
 * - url: identyfikator zasobu do zaczytania
 * - view: nazwa widoku w który mają być zaczytane dane 
 *   (jeśli puste to dane są zaczytane do aktualnego widoku).
 *
 * Następnie obiekt jest parsowany przez widok odpowiedzialny za jego wyświetlenie.
 * 
 */


var Main = {
	getUrl: function(uri) { return 'http://www.tvp.pl' + uri.replace(/&amp;/g, '&') + '&samsungwidget=1&v=2'; },
	//getUrl: function(uri) { return 'http://wczyzewski.v3.tvp.pl:8080' + uri.replace(/&amp;/g, '&') + '&samsungwidget=1&g7'; },
	getSearchUrl: function(query) { return '/pub/stat/websitelisting?object_id=2919829&child_mode=SIMPLE&rec_count=32&with_subdirs=true&link_as_copy=true&xslt=internet-tv/samsung/websites_listing.xslt&q=' + encodeURI(query) + '&5'; },
	//getSearchUrl: function(query) { return 'http://www.tvp.pl?query=' + encodeURI(query) + '&4'; },
	widgetAdapter: (Common.API.Widget) ? new Common.API.Widget() : {
		sendReadyEvent: function() { logger.debug('sendReadyEvent()'); },
		sendExitEvent: function() {},
		sendReturnEvent: function() {},
		blockNavigation: function(keyEvent) {},
		putInnerHTML: function(element, content) { 
			element.innerHTML = content
		},
	},
	tvKey: new Common.API.TVKeyValue(),
	userId: null,
	currentView: null,
	setCurrentView: function(viewSymbol) {
		this.currentView = Main.VIEWS[viewSymbol];
		return this.currentView;
	},
	init: function(global) { 
        this.VIEWS = global
    },
    deinit: function() {
        logger.debug('Main.deinit');
        Player.deinit();
    },
	exitApp: function() { 
        logger.debug('Main.exit');
        Main.widgetAdapter.sendExitEvent();
    },
    returnApp: function() {
        logger.debug('Main.returnApp');
        Main.widgetAdapter.sendReturnEvent();
    },
};

Main.start = function(view, url) {
	Main.UI.input.enable();
	Main.widgetAdapter.sendReadyEvent();

	logger.debug('loading menu');
	view.load(null, {url: url}, function(isSuccess) { 
		if (isSuccess) { 
			view.focus(); 
			view.loadSubview(); 
		}
	});
};

Main.loadView = function(parentView, data, callback) {
	if (data.view && Main.VIEWS[data.view]) {
		logger.debug('loading: ' + data.view);
		Main.VIEWS[data.view].load(parentView, data, callback);
	} else {
		logger.warn('brak widoku: ' + data.view + ' do załadowania urla: ' + data.url);
	}
};

Main.UI = {
	put: function(parent, content) {
		Main.widgetAdapter.putInnerHTML(parent, content);
	},
	videoProgress: {
		start: function() {
			if (!Main.UI.progress.isStarted) {
				Main.UI.progress.isStarted = true;
				DOMUtil.show('load');
				logger.debug('progress start ...');
			}
		},
		stop: function() {
			if (Main.UI.progress.isStarted) {
				Main.UI.progress.isStarted = false;
				logger.debug('progress stop ...');
				DOMUtil.hide('load');
			}
		},
	},
	progress: {
		isStarted: false,
		start: function(allowedActions) {
			if (!Main.UI.progress.isStarted) {
				Main.UI.progress.isStarted = true;
				Main.UI.input.disable(allowedActions);
				DOMUtil.show('load');
				logger.debug('progress start ...');
			}
		},
		stop: function() {
			if (Main.UI.progress.isStarted) {
				Main.UI.progress.isStarted = false;
				logger.debug('progress stop ...');
				Main.UI.input.enable();
				DOMUtil.hide('load');
			}
		},
	},

	dialog: {
        callback: null,
		error: function(msg) {
			logger.error('Dialog Error: ' + msg);
			Main.UI.put(DOMUtil.getElement('error-subject'), 'Błąd');
			Main.UI.put(DOMUtil.getElement('error-description'), msg);
			DOMUtil.show('error');
            Main.UI.dialog.setCallback(function() { DOMUtil.hide('error'); }, 3000);
		},
		info: function(msg) {
			logger.info('Dialog Info: ' + msg);
			Main.UI.put(DOMUtil.getElement('error-subject'), 'Informacja');
			Main.UI.put(DOMUtil.getElement('error-description'), msg);
			DOMUtil.show('error');
            Main.UI.dialog.setCallback(function() { DOMUtil.hide('error'); }, 2000);
		},
        clearCallback: function() {
            if (Main.UI.dialog.callback != null) {
                clearTimeout(Main.UI.dialog.callback.id);
                Main.UI.dialog.callback.func();
                Main.UI.dialog.callback = null;
            }
        },
        setCallback: function(func, timeout) {
            Main.UI.dialog.clearCallback();
            Main.UI.dialog.callback = {
                func: func,
                id: setTimeout(func, timeout),
            };
        },
        hide: function() {
            Main.UI.dialog.clearCallback();
        },
	},

	input: {
        disabled: false,
        allowedActions: null,
		disable: function(allowedActions) {
            Main.UI.input.disabled = true;
            Main.UI.input.allowedActions = allowedActions;
			//document.getElementById("anchor-void").focus();
			logger.debug('Main.UI.input.disable()');
		},
		enable: function() {
            Main.UI.input.disabled = false;
            Main.UI.input.allowedActions = null;
			logger.debug('Main.UI.input.enable()');
			document.getElementById("anchor").focus();
		},
		actionForCode: function(code) {
			var action = '';
			switch (code) {
				case Main.tvKey.KEY_PLAY: 
					action = 'play';
					break;
				case Main.tvKey.KEY_STOP:
				   action = 'stop';
					break;
				case Main.tvKey.KEY_PAUSE:
				   action = 'pause';
					break;
			    case Main.tvKey.KEY_FF:
				  	action =  'forward';
					break;
			    case Main.tvKey.KEY_RW:
					action = 'rewind';
					break;
				case Main.tvKey.KEY_BLUE:
					action = 'blue';
					break;
				case Main.tvKey.KEY_GREEN:
					action = 'green';
					break;
				case Main.tvKey.KEY_LEFT: 
					action = 'left';
					break;
				case Main.tvKey.KEY_RIGHT:
					action = 'right';
					break;
				case Main.tvKey.KEY_UP:
					action = 'up';
					break;
				case Main.tvKey.KEY_DOWN:
					action = 'down';
					break;
				case Main.tvKey.KEY_ENTER:
				case Main.tvKey.KEY_PANEL_ENTER:
					Main.widgetAdapter.blockNavigation(event);
					action = 'enter';
					break;
				case Main.tvKey.KEY_EXIT:
				case Main.tvKey.KEY_PANEL_EXIT:
					Main.widgetAdapter.blockNavigation(event);
					action = 'exit';
					break;
				case Main.tvKey.KEY_RETURN:
				case Main.tvKey.KEY_PANEL_RETURN:
					Main.widgetAdapter.blockNavigation(event);
					action = 'return';
					break;
				case Main.tvKey.KEY_VOL_UP:
				case Main.tvKey.KEY_PANEL_VOL_UP:
					VolumeBar.up();
					break;
				case Main.tvKey.KEY_VOL_DOWN:
				case Main.tvKey.KEY_PANEL_VOL_DOWN:
					VolumeBar.down();
					break;
				case Main.tvKey.KEY_MUTE:
					VolumeBar.mute();
					break;
				case Main.tvKey.KEY_1:
					action = '_1';
					break;
				case Main.tvKey.KEY_2:
					action = '_2';
					break;
				case Main.tvKey.KEY_3:
					action = '_3';
					break;
				case Main.tvKey.KEY_4:
					action = '_4';
					break;
				case Main.tvKey.KEY_5:
					action = '_5';
					break;
				case Main.tvKey.KEY_6:
					action = '_6';
					break;
				case Main.tvKey.KEY_7:
					action = '_7';
					break;
				case Main.tvKey.KEY_8:
					action = '_8';
					break;
				case Main.tvKey.KEY_9:
					action = '_9';
					break;
				case Main.tvKey.KEY_0:
					action = '_0';
					break;
				case Main.tvKey.KEY_PRECH:
					action = 'prech';
					break;
					
				default:
					logger.warn('Aplikacja nie obsługuje podanego kodu: ' + code);
			}
            Main.UI.dialog.hide();
			return action;
		},
        isBlocked: function(action) {
            logger.debug('allowedAction: ' + Main.UI.input.allowedAction);
            return Main.UI.input.allowedActions === undefined || Main.UI.input.allowedActions === null || Main.UI.input.allowedActions[action] === undefined;
        },
		dispatch: function(code) {
			var keyCode = code || event.keyCode;
			var action = Main.UI.input.actionForCode(keyCode);
			logger.debug("Key pressed: " + keyCode);
        
            // obsługa blokady
            if (Main.UI.input.disabled === true && Main.UI.input.isBlocked(action)) {
                logger.debug("Akcja zablokowana: " + action);
            } else {
                logger.debug("Akcja dozwolona: " + action);
                var view = Main.currentView;
                if (action != '') {
                    if (view.hasAction(action)) {
                        if (view.isReady()) {
                            logger.debug('Wywołanie akcji: ' + action + 'Action dla widoku: ' + view.name);
                            view[action + 'Action']();
                        } else {
                            logger.info('Zignorowane wywołanie akcji: ' + action + 'Action dla widoku: ' + view.name);
                        }
                    } else {
                        logger.warn('Aktualny widok : ' + view.name + ' nie obsługuje akcji: ' + action);
                    }
                }	
            }
		},
    }
};

var fileSystem = new FileSystem();

function Storage(type) {
	var type = type, data;

	function path() {
		return 'itvp_' + type + '_1.dat';
	}

	function getData() {
		logger.debug('Storage(' + type + ').getData(), data: ' + data);
		if (data == undefined) {
			data = readData();
		}
		return data;
	}

	function readFile() {
		logger.debug('Storage(' + type + ').getFile(), path: ' + path());
		var result = '', file;
		if (fileSystem.isValidCommonPath(path())) {
			file = fileSystem.openCommonFile(path(), 'r');
			if (file) {
				result = file.readAll();
				fileSystem.closeCommonFile(file);
			}
		}
		return result;
	}

	function readData() {
		var str = readFile();
		return (str == '') ? {} : JSON.parse(str);
	}

	function writeData(tmpData) {
		logger.debug('Storage(' + type + ').writeData()');
		var file = fileSystem.openCommonFile(path(), 'w');
		if (file) {
			data = tmpData;
			file.writeAll(JSON.stringify(tmpData));
			fileSystem.closeCommonFile(file);
			logger.info('Storage(' + type + ').writeData(): sukces.');
		} else {
			logger.info('Storage(' + type + ').writeData(): error.');
		}
	}

	this.all = function() {
		logger.debug('Storage(' + type + ').all()');
		var ar = [], tmp = getData();
		for (key in tmp) {
			logger.debug('key: ' + key + ', tmp[key]: ' + tmp[key]);
			ar.push(tmp[key]);
		}
		return ar;
	};

	this.get = function(id) {
		logger.debug('Storage(' + type + ').get(' + id + ')');
		return getData()[id];
	};

	this.add = function(obj) {
		var tmpData = getData();
		tmpData[obj.id] = obj;
		writeData(tmpData);
        if (obj.title) {
            Main.UI.dialog.info('Dodano: ' + obj.title);
        }
	};

	this.remove = function(obj) {
        if (obj.title) {
            Main.UI.dialog.info('Usunięto z ulubionych');
        }
		delete data[obj.id];
		writeData(data);
	};
	
}

Main.Storage = {
	Programs: new Storage('programs'),
	Video: new Storage('video'),
	init: function() {
		Main.Storage.Programs.all();
		Main.Storage.Video.all();
	}
};
