var FavoritesView = clone(View);
FavoritesView.init = function(name, options) {
	this._init(name, options);
	this.currentComponent = null;
	var pv = this;
	var tmpOptions = {
		onUnload: function(e) {
			FX.Grid.emptyElement(e);
			DOMUtil.hide(e);
		},
		onLoad: function(e, d) { 
			FX.Grid.fillElement(e, d);
		},
		onFocus: function(e, d) {
			DOMUtil.addClassName(e, 'focussed');
            pv.loadHelp();
		},
		onBlur: function(e, d) {
			DOMUtil.removeClassName(e, 'focussed');
		},
		onOverflow: function(up, down) {
			FX.Menu.onOverflowTop('favorites-programs-list-overflow-up', up);
			FX.Menu.onOverflowBottom('favorites-programs-list-overflow-down', down);
		},
		onTopBoundary: function() {
			pv.blur();
		},
	};
	this.programsList = new List(this.name + '-programs-list').init('item', tmpOptions);
	tmpOptions.onOverflow = function(up, down) {
		FX.Menu.onOverflowTop('favorites-video-list-overflow-up', up);
		FX.Menu.onOverflowBottom('favorites-video-list-overflow-down', down);
	};
	this.videoList = new List(this.name + '-video-list').init('item', tmpOptions);
};

FavoritesView.load = function(parentView, url, callback) {
	this.parentView = parentView;
	this.setCollections();
	if (callback && typeof callback == 'function') callback(true);
	this.show();
};

FavoritesView._focus = function() {
	this.setCollections();
	if (this.currentComponent != null) {
		this.currentComponent.focus();
	}
	this.show();
};

FavoritesView.reload = function() {
	this.programsList.blur();
	this.videoList.blur();
	this.setCollections();
	if (this.currentComponent != null) {
		this.currentComponent.focus();
	}
}

FavoritesView.setCollections = function() {
	FX.Menu.resetOverflow('favorites-programs-list-overflow-up');
	FX.Menu.resetOverflow('favorites-programs-list-overflow-down');
	FX.Menu.resetOverflow('favorites-video-list-overflow-up');
	FX.Menu.resetOverflow('favorites-video-list-overflow-down');
	this.programsList.load(Main.Storage.Programs.all(), -1);
	this.videoList.load(Main.Storage.Video.all(), -1);
	logger.debug('FavoritesView.setCollections(), this.videoList.isEmpty(): ' + this.videoList.isEmpty());
	logger.debug('FavoritesView.setCollections(), this.programsList.isEmpty(): ' + this.programsList.isEmpty());
	if (this.videoList.isEmpty()) {
		DOMUtil.hide('favorites-video-list');
		DOMUtil.show('favorites-video-empty');
	} else {
		DOMUtil.hide('favorites-video-empty');
		DOMUtil.show('favorites-video-list');
		this.currentComponent = this.videoList;
	}
	if (this.programsList.isEmpty()) {
		DOMUtil.hide('favorites-programs-list');
		DOMUtil.show('favorites-programs-empty');
	} else {
		DOMUtil.show('favorites-programs-list');
		DOMUtil.hide('favorites-programs-empty');
		this.currentComponent = this.programsList;
	}
};

FavoritesView.upAction = function() {
	if (this.currentComponent == this.videoList && !this.programsList.isEmpty()) {
		this.videoList.blur();
		this.currentComponent = this.programsList;
	}
	if (this.currentComponent != null) {
		this.currentComponent.focus();
	}
};

FavoritesView.downAction = function() {
	if (this.currentComponent == this.programsList && !this.videoList.isEmpty()) {
		this.programsList.blur();
		this.currentComponent = this.videoList;
	}
	if (this.currentComponent != null) {
		this.currentComponent.focus();
	}
};

FavoritesView.leftAction = function() {
	if (this.currentComponent != null) {
		this.currentComponent.prev();
	}
};

FavoritesView.rightAction = function() {
	if (this.currentComponent != null) {
		this.currentComponent.next();
	}
};

FavoritesView.getCurrent = function() {
	return this.currentComponent ? this.currentComponent.getCurrent() : null;
};

// usunięcie programu z ulubionych
FavoritesView.greenAction = function() {
	var obj = FavoritesView.getCurrent();
	if (Main.Storage.Programs.get(obj.data.id)) {
		Main.Storage.Programs.remove(obj.data);
		this.programsList.removeCurrent();
		if (this.programsList.isEmpty()) {
			DOMUtil.hide('favorites-programs-list');
			DOMUtil.show('favorites-programs-empty');
			if (this.videoList.isEmpty()) {
				this.blur();
			} else {
				this.programsList.blur();
				this.videoList.focus();
				this.currentComponent = this.videoList;
			}
		}
	}
};

// usunięcie video z ulubionych
FavoritesView.blueAction = function() {
	var obj = FavoritesView.getCurrent();
    if (Main.Storage.Video.get(obj.data.id)) {
		Main.Storage.Video.remove(obj.data);
		this.videoList.removeCurrent();
		if (this.videoList.isEmpty()) {
			DOMUtil.hide('favorites-video-list');
			DOMUtil.show('favorites-video-empty');
			if (this.programsList.isEmpty()) {
				this.blur();
			} else {
				this.videoList.blur();
				this.programsList.focus();
				this.currentComponent = this.programsList;
			}
		}
	}
};

FavoritesView.returnAction = function() {
	this.blur();
};

FavoritesView.blur = function() {
	this.videoList.blur();
	this.programsList.blur();
	this._blur();
};

FavoritesView.loadHelp = function() {
	var remove = '';
	var move = '';
    if (this.currentComponent == this.programsList && !this.programsList.isEmpty()) {
        remove = 'remove_serie';
    }
    if (this.currentComponent == this.videoList && !this.videoList.isEmpty()) {
        remove = 'remove';
    }

	if ((!this.videoList.isEmpty() || !this.programsList.isEmpty())) {
		move = 'move';
	}
	FX.Footer.load(remove, move, 'enter', 'ret');
};

FavoritesView.isReady = function() { return true; }
