var SearchView = clone(View);
SearchView.MODE = {
	QUERY: 0,
	BROWSE: 1,
};

SearchView.init = function(name, options) {
	Search.init('search-query-controls');
	this._init(name, options);
	this.isLoaded = true;
	this.mode = SearchView.MODE.QUERY;
	this.grid = new Grid(this.name + '-results-grid', 4, 2).init('item', {
		onUnload: function(e) {
			FX.Grid.emptyElement(e);
		},
		onLoad: function(e, d) { 
			FX.Grid.fillElement(e, d);
		},
		onFocus: function(e, d) {
			FX.Grid.elementFocus(e, d);
		},
		onBlur: function(e, d) {
			FX.Grid.elementBlur(e, d);
		},
	});
}

SearchView.load = function(parentView, url, callback) {
	Search.setMode(Search.LOWERCASE_MODE);
	this.parentView = parentView;
	if (callback) callback();
	this.show();
};


SearchView._0Action = function() {
	if (this.mode == SearchView.MODE.QUERY) {
		Search.push(0);
	}
};

SearchView._1Action = function() {
	if (this.mode == SearchView.MODE.QUERY) {
		Search.push(1);
	}
};

SearchView._2Action = function() {
	if (this.mode == SearchView.MODE.QUERY) {
		Search.push(2);
	}
};

SearchView._3Action = function() {
	if (this.mode == SearchView.MODE.QUERY) {
		Search.push(3);
	}
};

SearchView._4Action = function() {
	if (this.mode == SearchView.MODE.QUERY) {
		Search.push(4);
	}
};

SearchView._5Action = function() {
	if (this.mode == SearchView.MODE.QUERY) {
		Search.push(5);
	}
};

SearchView._6Action = function() {
	if (this.mode == SearchView.MODE.QUERY) {
		Search.push(6);
	}
};

SearchView._7Action = function() {
	if (this.mode == SearchView.MODE.QUERY) {
		Search.push(7);
	}
};

SearchView._8Action = function() {
	if (this.mode == SearchView.MODE.QUERY) {
		Search.push(8);
	}
};

SearchView._9Action = function() {
	if (this.mode == SearchView.MODE.QUERY) {
		Search.push(9);
	}
};

SearchView.blueAction = function() {
	if (this.mode == SearchView.MODE.QUERY) {
		Search.nextMode();
	}
};

SearchView.returnAction = function() {
	if (this.mode == SearchView.MODE.QUERY) {
		this.blur();
	} else {
		this.switchMode();
	}
};

SearchView.enterAction = function() {
	if (this.mode == SearchView.MODE.QUERY) {
		var url = Main.getSearchUrl(Search.query);
		Main.UI.progress.start();
		XHRUtil.send({
			url: url,
			onSuccess: function(xml) {
				SearchView.parseResults(xml);
			},
			onError: function() {
				Main.UI.dialog.error('Błąd połączenia.');
				logger.error('Błąd przy pobieraniu: ' + url);
			},
			onComplete: function(isSuccess) {
				Main.UI.progress.stop();
			},
		});
	} else {
		var tmp = this.grid.getCurrent();
		if (tmp != null) {
			Main.loadView(this, tmp.data, function(isSuccess) {
				if (isSuccess) {
					logger.info('load success: ' + tmp.data.url);
					Main.VIEWS[tmp.data.view].focus();
				} else {
					logger.info('load failed: ' + tmp.data.url);
				}
			});
		}
	}
};

SearchView.leftAction = function() {
	if (this.mode == SearchView.MODE.BROWSE && this.grid['left']) {
		this.grid.left();
	}
};

SearchView.rightAction = function() {
	if (this.mode == SearchView.MODE.BROWSE && this.grid['right']) {
		this.grid.right();
	}
};

SearchView.upAction = function() {
	if (this.mode == SearchView.MODE.BROWSE && this.grid['up']) {
		this.grid.up();
	}
};

SearchView.downAction = function() {
	if (this.mode == SearchView.MODE.BROWSE && this.grid['down']) {
		this.grid.down();
	}
};


SearchView.parseResults = function(xml) {
	var data = [];
	var responseDoc = xml.documentElement;
	if (responseDoc) {
		var itemElements = responseDoc.getElementsByTagName("object");
		for (var i = 0; i < itemElements.length; i++) {
			logger.debug('sparsowany object id: ' + itemElements[i].getAttribute('id'));
			data.push({
				id: itemElements[i].getAttribute('id'), 
				url: itemElements[i].getAttribute('url'), 
				view: itemElements[i].getAttribute('view'), 
				title: DOMUtil.getChildText(itemElements[i].getElementsByTagName('title'), ''),
				img_src: DOMUtil.getChildAttribute(itemElements[i].getElementsByTagName('img'), 'src', ''),
			});
		}
		this.grid.load(data, -1);
		this.switchMode();
		if (data.length == 0) {
			DOMUtil.hide(this.name + '-results-ok');
			DOMUtil.show(this.name + '-results-none');
		} else {
			DOMUtil.hide(this.name + '-results-none');
			DOMUtil.show(this.name + '-results-ok');
			this.grid.focus();
		}
	} else {
		Main.UI.dialog.error('Nie powiodło się pobranie danych.');
	}
	return this;
};

SearchView.switchMode = function() {
	if (this.mode == SearchView.MODE.QUERY) {
		this.mode = SearchView.MODE.BROWSE;
		DOMUtil.hide(this.name + '-query');
		DOMUtil.show(this.name + '-results');
	} else {
		this.mode = SearchView.MODE.QUERY;
		DOMUtil.hide(this.name + '-results');
		DOMUtil.show(this.name + '-query');
	}
	this.loadHelp();
};

// Wykasowanie ostatniego znaku.
SearchView.prechAction = function() {
	if (this.mode == SearchView.MODE.QUERY) {
		Search.pop();
	}
};

SearchView.isReady = function() { return this.isLoaded; };

SearchView.loadHelp = function() {
	if (this.mode == SearchView.MODE.QUERY) {
		FX.Footer.load('mode', 'input', 'search');
	} else {
		FX.Footer.load('move', 'enter', 'ret');
	}
};

