var VideoView = clone(View);
var VideoMode = { FULLSCREEN: 0, EMBED: 1};
VideoView.init = function(name, options) {
	this._init(name, options);
	this.mode = VideoMode.EMBED;
	this.titleElement = DOMUtil.getElement(this.name + '-title'); 
	this.prevButtonElement = DOMUtil.getElement(this.name + '-prev-button'); 
	this.nextButtonElement = DOMUtil.getElement(this.name + '-next-button'); 
	this.currentComponent = null;
	this.data = {};

	var pv = this;
	this.textField = new TextField(this.name + '-description', pv, {scrollHeight: 360}).init({});

    this.hideCallback; // funkcja ukrywajaca kontrolki przy fullscreenie

	return this;
};

VideoView.parse = function(xml) {
	logger.debug('VideoView.parse');
	var responseDoc = xml.documentElement, itemElements, i;
	var that = this;
	if (responseDoc) {
		itemElements = responseDoc.getElementsByTagName("object");
		for (i = 0; i < itemElements.length; i++) {
			this.data = {
				id: itemElements[i].getAttribute('id'), 
				title: DOMUtil.getChildText(itemElements[i].getElementsByTagName('title'), ''),
				url: itemElements[i].getAttribute('url'), 
				video_url: itemElements[i].getAttribute('video_url'), 
				prev_url: itemElements[i].getAttribute('prev_url'), 
				next_url: itemElements[i].getAttribute('next_url'), 
				view: itemElements[i].getAttribute('view'), 
				description: DOMUtil.getChildText(itemElements[i].getElementsByTagName('description'), ''),
				img_src: DOMUtil.getChildAttribute(itemElements[i].getElementsByTagName('img'), 'src', ''),
			};
			break;
		}
		Main.UI.put(this.titleElement, this.data.title);
		this.textField.load(this.data.description, VideoView.loadHelp);
		if (this.data.prev_url) {
		   // DOMUtil.toggleClassName(this.prevButtonElement, 'inactive', 'active');
		} else {
		   // DOMUtil.toggleClassName(this.prevButtonElement, 'active', 'inactive');
		}
		if (this.data.next_url) {
		   // DOMUtil.toggleClassName(this.nextButtonElement, 'inactive', 'active');
		} else {
		   // DOMUtil.toggleClassName(this.nextButtonElement, 'active', 'inactive');
		}
		Player.onStop = function() {
			logger.debug('Player.onStop()');
			if (that.mode == VideoMode.FULLSCREEN) {
				that.embedMode();
			}
		};
		Player.setVideoURL(this.data.video_url, this.data.title, this.data.id);
	} else {
		Main.UI.dialog.error('Nie powiodło się pobranie danych.');
	}
	return this;
};

VideoView.reload = function(url, callback) {
	logger.debug('VideoView.reload: ' + url);
	Player.stopVideo();
	Main.UI.progress.start();
	XHRUtil.send({
		url: url,
		onSuccess: function(xml) {
			VideoView.parse(xml);
			VideoView.currentComponent = this.textField;
			if (callback) {
				callback();
			}
			VideoView.loadHelp();
		},
		onError: function() {
			Main.UI.dialog.error('Błąd połączenia.');
			logger.error('Błąd przy pobieraniu: ' + url);
		},
		onComplete: function(isSuccess) {
            Main.UI.progress.stop();
            VideoView.textField.focus();
		},
	});
};

VideoView.showControlls = function(hide) {
    if (this.mode == VideoMode.FULLSCREEN) {
        clearTimeout(this.hideCallback);
        if (hide) {
            this.hideCallback = setTimeout(function() {
                DOMUtil.hide('fullscreen-overlay');
                DOMUtil.hide('fullscreen-footer');
            }, 3000);
        }
        DOMUtil.show('fullscreen-overlay', true);
        DOMUtil.show('fullscreen-footer');
    }
};
VideoView.leftAction = function() {
	if (this.mode == VideoMode.EMBED && this.data.prev_url) {
		VideoView.reload(this.data.prev_url, function() {
			if (VideoView.parentView == ProgramView) {
				ProgramView.grid.prev();
			}
			Player.playVideo();
		});
	}
};

VideoView.rightAction = function() {
	if (this.mode == VideoMode.EMBED && this.data.next_url) {
		VideoView.reload(this.data.next_url, function() {
			if (VideoView.parentView == ProgramView) {
				ProgramView.grid.next();
			}
			Player.playVideo();
		});
	}
};

VideoView.playAction = function() {
    VideoView.showControlls(true);
	Player.playVideo();
};

VideoView.pauseAction = function() {
    VideoView.showControlls(false);
	Player.pauseVideo();
};

VideoView.stopAction = function() {
    VideoView.showControlls(true);
	Player.stopVideo();
};

VideoView.forwardAction = function() {
    VideoView.showControlls(true);
	Player.skipForwardVideo();
};

VideoView.rewindAction = function() {
    VideoView.showControlls(true);
	Player.skipBackwardVideo();
};

VideoView._focus = function() {
	this.currentComponent = this.textField;
	this.textField.focus();
	this.embedMode();
	this.playAction();
};

VideoView.loadHelp = function() {
	var fav = 'fav_' + ((Main.Storage.Video.get(VideoView.data.id)) ? 'remove' : 'add'); 
	var prev = VideoView.data.prev_url ? 'prev' : '';
	var next = VideoView.data.next_url ? 'next' : '';
	var scrl = VideoView.textField.hasScroll() ? 'scrl' : '';
	FX.Footer.load(fav, prev, next, scrl, 'ff_rev', 'full', 'ret');
	FX.FullscreenFooter.load(fav, 'ff_rev', 'full', 'ret');
};

VideoView.blueAction = function() {
    VideoView.showControlls(true);
	if (Main.Storage.Video.get(this.data.id)) {
		Main.Storage.Video.remove(this.data);
	} else {
		Main.Storage.Video.add(this.data);
	}
	FavoritesView.reload();
	VideoView.loadHelp();
};

VideoView.returnAction = function() {
	logger.debug('VideoView.returnAction, mode: ' + this.mode);
	if (this.mode == VideoMode.EMBED) {
		//Player.stopVideo();
		//Player.setHiddenMode();
		this.blur();
	} else {
		this.mode = VideoMode.EMBED;
		DOMUtil.show(this.element);
		DOMUtil.hideAndShowSiblings('fullscreen');
        DOMUtil.toggleClassName(document.getElementsByTagName('body')[0], 'fullscreen', 'player');
		Player.setWindowMode();
	}
};

VideoView.enterAction = function() {
	logger.debug('VideoView.enterAction, mode: ' + this.mode);
	if (this.mode == VideoMode.EMBED) {
		this.fullscreenMode();
	} else {
        this.embedMode();
    }	
};

VideoView.fullscreenMode = function() {
	this.mode = VideoMode.FULLSCREEN;
	DOMUtil.hide(this.element);
	DOMUtil.showAndHideSiblings('fullscreen');
	DOMUtil.toggleClassName(document.getElementsByTagName('body')[0], 'player', 'fullscreen');
	Player.setFullscreenMode();
    VideoView.showControlls(true);
};

VideoView.embedMode = function() {
	this.mode = VideoMode.EMBED;
	DOMUtil.show(this.element);
	DOMUtil.hideAndShowSiblings('fullscreen');
	DOMUtil.toggleClassName(document.getElementsByTagName('body')[0], 'fullscreen', 'player');
	Player.setWindowMode();
};

VideoView.blur = function() {
    logger.debug('VideoView.blur');
	this.textField.blur();
	Player.stopVideo();
	Player.setHiddenMode();
	DOMUtil.removeClassName(document.getElementsByTagName('body')[0], 'player');
	this.hide(true);
	this._blur();
};

VideoView.isReady = function() { return true; };
