var View = {
	// metoda init powinna być wołana po załadowaniu całego dokumentu
	init: function(name, options) {
		return this._init(name, options);
	},

	_init: function(name, options) {
		var options = options || {};

		this.name = name;
		this.element = DOMUtil.getElement(name);
		this.data = {};
        this.url = null;
		this._isReady = false;
	   	this._isVisited = false;

		if (!this.element) { throw 'Brak elementu: ' + name + ' w DOM'; }


	   	// zdarzenia występujące przy przełączaniu okien
		this.onFocus = options.onFocus || function() {};
		this.onBlur = options.onBlur || function() {};
		return this;
	},

	loadData: function(parentView, data, options) {
	},

	unLoad: function() {},

	load: function(parentView, data, callback) {
		this._load(parentView, data, callback);
	},

	// właściwe załadowanie danych, przy pomocy XHR
	_load: function(parentView, data, callback) {
		this.parentView = parentView;
		if (data.url && this.url != data.url) {
			var that = this;
			XHRUtil.send({
				url: data.url,
				allowedActions: data.allowedActions,
				onSuccess: function(xml) {
                    that._isReady = false;
					that.data = data;
                    that.url = data.url;
					that.unLoad();
					that.parse(xml);
					that._isVisited = false;
					that.show();
				},
				onError: function() {
					Main.UI.dialog.error('Błąd połączenia.');
					logger.error('Błąd przy pobieraniu: ' + data.url);
				},
				onComplete: function(isSuccess) {
					Main.UI.progress.stop();
					that._isReady = true;
					if (callback && typeof callback == 'function') callback(isSuccess); 
				},
			});
		} else {
			this.show();
			if (callback && typeof callback == 'function') callback(true);
		}
		return this;
	},
	
	show: function() {
		logger.debug('View.show: ' + this.name);
		DOMUtil.showAndHideSiblings(this.element);
		return this;
	},

	hide: function() {
		logger.debug('View.hide: ' + this.name);
		DOMUtil.hideAndShowSiblings(this.element);
		return this;
	},

	/**
	 * Metoda zwraca aktualnie wybrany obiekt: {element, data}
	 */
	getCurrent: function() { return null; },

	/**
	 * Ustawienie focusa (do tego widoku będą przesyłane akcje z pilota).
	 *
	 * Dodatkowo żeby móc zachować stan komponentów mimo zmiany focusa (okienko
	 * zostało przesunięte w tło), przechowywana jest informacja o dotychczasowych
	 * interakcjach.
	 */
	focus: function() {
		logger.debug('View.focus - name: ' + this.name + ', url: ' + this.url);
		if (this.isReady()) {
			if (this.isVisited() == false) {
				this._isVisited = true;
				this._focus();
			}
			this.show();
			this.loadHelp();
			Main.currentView = this;
		} else {
            logger.debug('is not ready');
        }
		return this;
	},

	/**
	 * Właściwa metoda obsługująca inicjującego focusa, np. ustawienie 
	 * focusa na odpowiedni podelement po załadowaniu nowego okna.
	 */
	_focus: function() {
		return this;
	},

	blur: function() {
		this._blur();
	},

	/**
	 * Właściwa metoda odpowiadająca za przeniesienie focusa na okno wywołujące.
	 */
	_blur: function() {
		logger.debug('View._blur: ' + this.name);
		this._isVisited = false;
		this.parentView.focus();
		return this;
	},

	select: function() {
		return this;
	},

	/**
	 * Sprawdzenie czy wszystkie elementy widoku zostały załadowane
	 * i czy może być interakcja z widokiem.
	 */
	isReady: function() {
		return this._isReady;
	},

	isVisited: function() {
		return this._isVisited;
	},


	/**
	 * Sprawdzenie jakie akcje obsługuje dany widok.
	 */
	hasAction: function(actionName) {
		return this[actionName + 'Action'];
	},

	/**
	 * Standardowa operacja dla większości widoków: załadowanie na podstawie
	 * wybranego widoku nowego okna.
	 */
	enterAction: function() {
		var tmp = this.getCurrent();
		if (tmp != null) {
            logger.debug('tmp.msg: ' + tmp.data.msg);
            if (tmp.data.msg != undefined && tmp.data.msg.length > 0) {
                Main.UI.dialog.info(tmp.data.msg);
            } else {
                Main.loadView(this, tmp.data, function(isSuccess) {
                    if (isSuccess) {
                        logger.info('load success: ' + tmp.data.url);
                        Main.VIEWS[tmp.data.view].focus();
                    } else {
                        logger.info('load failed: ' + tmp.data.url);
                    }
                });
            }
		}
		return this;
	},

	loadHelp: function() {
		FX.Footer.load();
	},

    exitAction: function() {
        Main.exitApp();
    },
};
